@@ -105,7 +105,7 @@ class RetireAction(Action):
         """Retire the context item."""
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         # if we want to view the index template now, then unset the itemid
         # context info (a special-case for retire actions on the index page)
@@ -284,7 +284,7 @@ class EditCSVAction(Action):
         """
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         # figure the properties list for the class
         cl = self.db.classes[self.classname]
@@ -539,9 +539,25 @@ class EditCommon(Action):
         Base behaviour is to check the user can edit this class. No additional
         property checks are made.
         """
+
         if not classname :
             classname = self.client.classname
-        return self.hasPermission('Create', classname=classname)
+        
+        if not self.hasPermission('Create', classname=classname):
+            return 0
+
+        # Check Edit permission for each property, to avoid being able
+        # to set restricted ones on new item creation
+        for key in props:
+            if not self.hasPermission('Edit', classname=classname,
+                                      property=key):
+                # We restrict by default and special-case allowed properties
+                if key == 'date' or key == 'content':
+                    continue
+                elif key == 'author' and props[key] == self.userid:
+                    continue
+                return 0
+        return 1
 
 class EditItemAction(EditCommon):
     def lastUserActivity(self):
@@ -587,7 +603,7 @@ class EditItemAction(EditCommon):
         """
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         user_activity = self.lastUserActivity()
         if user_activity:
@@ -633,7 +649,7 @@ class NewItemAction(EditCommon):
         '''
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         # parse the props from the form
         try:
@@ -643,11 +659,6 @@ class NewItemAction(EditCommon):
                 % str(message))
             return
 
-        # guard against new user creation that would bypass security checks
-        for key in props:
-            if 'user' in key:
-                return
-
         # handle the props - edit or create
         try:
             # when it hits the None element, it'll set self.nodeid
@@ -809,7 +820,7 @@ class ConfRegoAction(RegoCommon):
 
 class RegisterAction(RegoCommon, EditCommon):
     name = 'register'
-    permissionType = 'Create'
+    permissionType = 'Register'
 
     def handle(self):
         """Attempt to create a new user based on the contents of the form
@@ -819,7 +830,7 @@ class RegisterAction(RegoCommon, EditCommon):
         """
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         # parse the props from the form
         try:
@@ -937,7 +948,7 @@ class LoginAction(Action):
         """
         # ensure modification comes via POST
         if self.client.env['REQUEST_METHOD'] != 'POST':
-            self.client.error_message.append(self._('Invalid request'))
+            raise roundup.exceptions.Reject(self._('Invalid request'))
 
         # we need the username at a minimum
         if not self.form.has_key('__login_name'):
@@ -1041,7 +1052,7 @@ class ExportCSVAction(Action):
             row = []
             for name in columns:
                 # check permission to view this property on this item
-                if exists and not self.hasPermission('View', itemid=itemid,
+                if not self.hasPermission('View', itemid=itemid,
                         classname=request.classname, property=name):
                     raise exceptions.Unauthorised, self._(
                         'You do not have permission to view %(class)s'
@@ -1086,11 +1097,11 @@ class Bridge(BaseAction):
     def permission(self, args):
         """Raise Unauthorised if the current user is not allowed to execute
         this action. Users may override this method."""
-        
+
         pass
 
     def handle(self, args):
-        
+
         raise NotImplementedError
 
 # vim: set filetype=python sts=4 sw=4 et si :
