@@ -14,7 +14,7 @@ from roundup.cgi.actions import *
 from roundup.exceptions import *
 from roundup.cgi.exceptions import *
 from roundup.cgi.form_parser import FormParser
-from roundup.mailer import Mailer, MessageSendError
+from roundup.mailer import Mailer, MessageSendError, encode_quopri
 from roundup.cgi import accept_language
 from roundup import xmlrpc
 
@@ -387,7 +387,6 @@ class Client:
                                            self.translator,
                                            allow_none=True)
         output = handler.dispatch(input)
-        self.db.commit()
 
         self.setHeader("Content-Type", "text/xml")
         self.setHeader("Content-Length", str(len(output)))
@@ -721,6 +720,10 @@ class Client:
             else:
                 self.db.close()
                 self.db = self.instance.open(username)
+                # The old session API refers to the closed database;
+                # we can no longer use it.
+                self.session_api = Session(self)
+ 
 
     def determine_context(self, dre=re.compile(r'([^\d]+)0*(\d+)')):
         """Determine the context of this page from the URL:
@@ -991,13 +994,13 @@ class Client:
                 to = [self.mailer.config.ADMIN_EMAIL]
                 subject = "Templating Error: %s" % exc_info[1]
                 content = cgitb.pt_html()
-                message, writer = self.mailer.get_standard_message(
-                    to, subject)
-                writer.addheader('Content-Transfer-Encoding', 'quoted-printable')
-                body = writer.startbody('text/html; charset=utf-8')
-                content = StringIO(content)
-                quopri.encode(content, body, 0)
-                self.mailer.smtp_send(to, message)
+                message = self.mailer.get_standard_message(to, subject)
+                # delete existing content-type headers
+                del message['Content-type']
+                message['Content-type'] = 'text/html; charset=utf-8'
+                message.set_payload(content)
+                encode_quopri(message)
+                self.mailer.smtp_send(to, str(message))
                 # Now report the error to the user.
                 return self._(error_message)
             except:
@@ -1323,7 +1326,7 @@ class Client:
             # RFC 2616 14.13: Content-Length
             #
             # Tell the client how much data we are providing.
-            self.setHeader("Content-Length", length)
+            self.setHeader("Content-Length", str(length))
             # Send the HTTP header.
             self.header()
         # If the client doesn't actually want the body, or if we are
