@@ -1,7 +1,6 @@
 """Sending Roundup-specific mail over SMTP.
 """
 __docformat__ = 'restructuredtext'
-# $Id: mailer.py,v 1.22 2008-07-21 01:44:58 richard Exp $
 
 import time, quopri, os, socket, smtplib, re, sys, traceback, email
 
@@ -23,6 +22,7 @@ def encode_quopri(msg):
     orig = msg.get_payload()
     encdata = quopri.encodestring(orig)
     msg.set_payload(encdata)
+    del msg['Content-Transfer-Encoding']
     msg['Content-Transfer-Encoding'] = 'quoted-printable'
 
 class Mailer:
@@ -55,7 +55,7 @@ class Mailer:
         Subject and author are encoded using the EMAIL_CHARSET from the
         config (default UTF-8).
 
-        Returns a Message object and body part writer.
+        Returns a Message object.
         '''
         # encode header values if they need to be
         charset = getattr(self.config, 'EMAIL_CHARSET', 'utf-8')
@@ -69,9 +69,8 @@ class Mailer:
         if multipart:
             message = MIMEMultipart()
         else:
-            message = Message()
+            message = MIMEText("")
             message.set_charset(charset)
-            message['Content-Type'] = 'text/plain; charset="%s"'%charset
 
         try:
             message['Subject'] = subject.encode('ascii')
@@ -98,8 +97,6 @@ class Mailer:
         # finally, an aid to debugging problems
         message['X-Roundup-Version'] = __version__
 
-        message['MIME-Version'] = '1.0'
-
         return message
 
     def standard_message(self, to, subject, content, author=None):
@@ -115,7 +112,8 @@ class Mailer:
         """
         message = self.get_standard_message(to, subject, author)
         message.set_payload(content)
-        self.smtp_send(to, str(message))
+        encode_quopri(message)
+        self.smtp_send(to, message.as_string())
 
     def bounce_message(self, bounced_message, to, error,
                        subject='Failed issue tracker submission'):
@@ -139,29 +137,30 @@ class Mailer:
         elif error_messages_to == "both":
             to.append(dispatcher_email)
 
-        message = self.get_standard_message(to, subject)
+        message = self.get_standard_message(to, subject, multipart=True)
 
         # add the error text
-        part = MIMEText(error)
+        part = MIMEText('\n'.join(error))
         message.attach(part)
 
         # attach the original message to the returned message
+        body = []
+        for header in bounced_message.headers:
+            body.append(header)
         try:
             bounced_message.rewindbody()
-        except IOError, message:
-            body.write("*** couldn't include message body: %s ***"
-                       % bounced_message)
+        except IOError, errmessage:
+            body.append("*** couldn't include message body: %s ***" %
+                errmessage)
         else:
-            body.write(bounced_message.fp.read())
-        part = MIMEText(bounced_message.fp.read())
-        part['Content-Disposition'] = 'attachment'
-        for header in bounced_message.headers:
-            part.write(header)
+            body.append('\n')
+            body.append(bounced_message.fp.read())
+        part = MIMEText(''.join(body))
         message.attach(part)
 
         # send
         try:
-            self.smtp_send(to, str(message))
+            self.smtp_send(to, message.as_string())
         except MessageSendError:
             # squash mail sending errors when bouncing mail
             # TODO this *could* be better, as we could notify admin of the
@@ -212,6 +211,7 @@ class SMTPConnection(smtplib.SMTP):
 
         # start the TLS if requested
         if config["MAIL_TLS"]:
+            self.ehlo()
             self.starttls(config["MAIL_TLS_KEYFILE"],
                 config["MAIL_TLS_CERTFILE"])
 
